/*******************************************************************************
* File Name: BT_Receiver_INT.c
* Version 2.30
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
* Note:
*  Any unusual or non-standard behavior should be noted here. Other-
*  wise, this section should remain blank.
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "BT_Receiver.h"
#include "CyLib.h"


/***************************************
* Custom Declratations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if( (BT_Receiver_RX_ENABLED || BT_Receiver_HD_ENABLED) && \
     (BT_Receiver_RXBUFFERSIZE > BT_Receiver_FIFO_LENGTH))


    /*******************************************************************************
    * Function Name: BT_Receiver_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  BT_Receiver_rxBuffer - RAM buffer pointer for save received data.
    *  BT_Receiver_rxBufferWrite - cyclic index for write to rxBuffer,
    *     increments after each byte saved to buffer.
    *  BT_Receiver_rxBufferRead - cyclic index for read from rxBuffer,
    *     checked to detect overflow condition.
    *  BT_Receiver_rxBufferOverflow - software overflow flag. Set to one
    *     when BT_Receiver_rxBufferWrite index overtakes
    *     BT_Receiver_rxBufferRead index.
    *  BT_Receiver_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when BT_Receiver_rxBufferWrite is equal to
    *    BT_Receiver_rxBufferRead
    *  BT_Receiver_rxAddressMode - this variable contains the Address mode,
    *     selected in customizer or set by UART_SetRxAddressMode() API.
    *  BT_Receiver_rxAddressDetected - set to 1 when correct address received,
    *     and analysed to store following addressed data bytes to the buffer.
    *     When not correct address received, set to 0 to skip following data bytes.
    *
    *******************************************************************************/
    CY_ISR(BT_Receiver_RXISR)
    {
        uint8 readData;
        uint8 increment_pointer = 0u;
        #if(CY_PSOC3)
            uint8 int_en;
        #endif /* CY_PSOC3 */

        /* User code required at start of ISR */
        /* `#START BT_Receiver_RXISR_START` */

        /* `#END` */

        #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
            int_en = EA;
            CyGlobalIntEnable;
        #endif /* CY_PSOC3 */

        readData = BT_Receiver_RXSTATUS_REG;

        if((readData & (BT_Receiver_RX_STS_BREAK | BT_Receiver_RX_STS_PAR_ERROR |
                        BT_Receiver_RX_STS_STOP_ERROR | BT_Receiver_RX_STS_OVERRUN)) != 0u)
        {
            /* ERROR handling. */
            /* `#START BT_Receiver_RXISR_ERROR` */

            /* `#END` */
        }

        while((readData & BT_Receiver_RX_STS_FIFO_NOTEMPTY) != 0u)
        {

            #if (BT_Receiver_RXHW_ADDRESS_ENABLED)
                if(BT_Receiver_rxAddressMode == (uint8)BT_Receiver__B_UART__AM_SW_DETECT_TO_BUFFER)
                {
                    if((readData & BT_Receiver_RX_STS_MRKSPC) != 0u)
                    {
                        if ((readData & BT_Receiver_RX_STS_ADDR_MATCH) != 0u)
                        {
                            BT_Receiver_rxAddressDetected = 1u;
                        }
                        else
                        {
                            BT_Receiver_rxAddressDetected = 0u;
                        }
                    }

                    readData = BT_Receiver_RXDATA_REG;
                    if(BT_Receiver_rxAddressDetected != 0u)
                    {   /* store only addressed data */
                        BT_Receiver_rxBuffer[BT_Receiver_rxBufferWrite] = readData;
                        increment_pointer = 1u;
                    }
                }
                else /* without software addressing */
                {
                    BT_Receiver_rxBuffer[BT_Receiver_rxBufferWrite] = BT_Receiver_RXDATA_REG;
                    increment_pointer = 1u;
                }
            #else  /* without addressing */
                BT_Receiver_rxBuffer[BT_Receiver_rxBufferWrite] = BT_Receiver_RXDATA_REG;
                increment_pointer = 1u;
            #endif /* End SW_DETECT_TO_BUFFER */

            /* do not increment buffer pointer when skip not adderessed data */
            if( increment_pointer != 0u )
            {
                if(BT_Receiver_rxBufferLoopDetect != 0u)
                {   /* Set Software Buffer status Overflow */
                    BT_Receiver_rxBufferOverflow = 1u;
                }
                /* Set next pointer. */
                BT_Receiver_rxBufferWrite++;

                /* Check pointer for a loop condition */
                if(BT_Receiver_rxBufferWrite >= BT_Receiver_RXBUFFERSIZE)
                {
                    BT_Receiver_rxBufferWrite = 0u;
                }
                /* Detect pre-overload condition and set flag */
                if(BT_Receiver_rxBufferWrite == BT_Receiver_rxBufferRead)
                {
                    BT_Receiver_rxBufferLoopDetect = 1u;
                    /* When Hardware Flow Control selected */
                    #if(BT_Receiver_FLOW_CONTROL != 0u)
                    /* Disable RX interrupt mask, it will be enabled when user read data from the buffer using APIs */
                        BT_Receiver_RXSTATUS_MASK_REG  &= (uint8)~BT_Receiver_RX_STS_FIFO_NOTEMPTY;
                        CyIntClearPending(BT_Receiver_RX_VECT_NUM);
                        break; /* Break the reading of the FIFO loop, leave the data there for generating RTS signal */
                    #endif /* End BT_Receiver_FLOW_CONTROL != 0 */
                }
            }

            /* Check again if there is data. */
            readData = BT_Receiver_RXSTATUS_REG;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START BT_Receiver_RXISR_END` */

        /* `#END` */

        #if(CY_PSOC3)
            EA = int_en;
        #endif /* CY_PSOC3 */

    }

#endif /* End BT_Receiver_RX_ENABLED && (BT_Receiver_RXBUFFERSIZE > BT_Receiver_FIFO_LENGTH) */


#if(BT_Receiver_TX_ENABLED && (BT_Receiver_TXBUFFERSIZE > BT_Receiver_FIFO_LENGTH))


    /*******************************************************************************
    * Function Name: BT_Receiver_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  BT_Receiver_txBuffer - RAM buffer pointer for transmit data from.
    *  BT_Receiver_txBufferRead - cyclic index for read and transmit data
    *     from txBuffer, increments after each transmited byte.
    *  BT_Receiver_rxBufferWrite - cyclic index for write to txBuffer,
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(BT_Receiver_TXISR)
    {

        #if(CY_PSOC3)
            uint8 int_en;
        #endif /* CY_PSOC3 */

        /* User code required at start of ISR */
        /* `#START BT_Receiver_TXISR_START` */

        /* `#END` */

        #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
            int_en = EA;
            CyGlobalIntEnable;
        #endif /* CY_PSOC3 */

        while((BT_Receiver_txBufferRead != BT_Receiver_txBufferWrite) &&
             ((BT_Receiver_TXSTATUS_REG & BT_Receiver_TX_STS_FIFO_FULL) == 0u))
        {
            /* Check pointer. */
            if(BT_Receiver_txBufferRead >= BT_Receiver_TXBUFFERSIZE)
            {
                BT_Receiver_txBufferRead = 0u;
            }

            BT_Receiver_TXDATA_REG = BT_Receiver_txBuffer[BT_Receiver_txBufferRead];

            /* Set next pointer. */
            BT_Receiver_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START BT_Receiver_TXISR_END` */

        /* `#END` */

        #if(CY_PSOC3)
            EA = int_en;
        #endif /* CY_PSOC3 */

    }

#endif /* End BT_Receiver_TX_ENABLED && (BT_Receiver_TXBUFFERSIZE > BT_Receiver_FIFO_LENGTH) */


/* [] END OF FILE */
